/*
    CharityWorld: 
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import "ExtMeanSimpleAgent.h"
#import "Verbosity.h"
#import "DoubleSimple.h"

@implementation ExtMeanSimpleAgent

/*

computeAverageWealth

Computes the local average wealth excluding the agent itself
-- the average of the wealth of its neighbours only. If the agent's wealth
is (less than | equal to | greater than) the local average calculated as in
Agent.m, then the same relationship should remain when the average is
computed as it is in this class. If it does not, it is due to the 
presence of some artefact (floating-point errors?)

*/

-(void)computeLocalWealth {
  id <Index> ix;
  Agent *nbr;

  [localWealth cset: 0.0];
  [minimum set: [maximum set: wealth]];

  if([Verbosity showAgentAverage]) {
    printf("Examining neighbouring wealth. I'm not computing my wealth");
    printf("\n\tTotal = ");
    [localWealth writeAccurately: stdout];
    printf(", minimum = ");
    [minimum writeAccurately: stdout];
    printf(", maximum = ");
    [maximum writeAccurately: stdout];
    printf("\n");
  }
  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    DoubleSimple *nbrWealth = [nbr getWealth];
    [localWealth add: nbrWealth];
    [minimum set: [minimum gt: nbrWealth] ? nbrWealth : minimum];
    [maximum set: [maximum lt: nbrWealth] ? nbrWealth : maximum];
    if([Verbosity showAgentAverage]) {
      printf("\tNeighbouring wealth: ");
      [nbrWealth writeAccurately: stdout];
      printf("\n\tTotal = ");
      [localWealth writeAccurately: stdout];
      printf(", minimum = ");
      [minimum writeAccurately: stdout];
      printf(", maximum = ");
      [maximum writeAccurately: stdout];
      printf("\n");
    }
    [nbrWealth free];
  }
  [ix drop];
  [localWealth idiv: (numberOfNbrs)];

  if([Verbosity showAgentAverage]) {
    printf("\tExternal Local Wealth (not including my wealth)= ");
    [localWealth writeAccurately: stdout];
    printf("\n");
  }
}

@end
