/*
    CharityWorld: ObserverSwarm.h
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import <simtoolsgui.h>
#import <analysis.h>
#import <simtoolsgui/GUISwarm.h>
#import <space.h>
#import <defobj.h>

@class ModelSwarm, PNGRaster;

@interface ObserverSwarm: GUISwarm {
  int displayFrequency;
  id displayActions;
  id displaySchedule;
  id outerSchedule;
  BOOL hideWealthMean;
  BOOL hideDonorWealthMean;
  BOOL hideBeggarWealthMean;
  BOOL hideWealthMax;
  BOOL hideWealthMin;
  BOOL hideScheduleBeggars;
  BOOL hideScheduleDonors;
  BOOL hideIlogbRaster;

  BOOL scheduleGraphFileOutput;
  const char *scheduleGraphFileName;

  BOOL bwrasters;

  id model;

  id archiver;

  id <Colormap> colormap;
  id <Colormap> ilogbmap;
  id <ZoomRaster> wealthRaster;
  PNGRaster *wealthPNG;
  int saveWealthPNG;
  int saveWealthPNGInterval;
  const char *wealthPNGStem;
  id <ZoomRaster> localWealthRaster;
#ifndef __CYGWIN__
  id <ZoomRaster> fpclassRaster;
#endif
  id <ZoomRaster> ilogbRaster;
  id <ZoomRaster> scheduleRaster;

  id <EZGraph> wealthGraph;
  id <EZGraph> beggarGraph;
  id <EZGraph> zoneGraph;
  id <EZBin> beggarDist;
  id <EZBin> donorDist;

  id <Object2dDisplay> wealthDisplay;
  id <Object2dDisplay> donationTraceDisplay;
  id <Object2dDisplay> localWealthDisplay;
#ifndef __CYGWIN__
  id <Object2dDisplay> fpclassDisplay;
#endif
  id <Object2dDisplay> ilogbDisplay;
  id <Object2dDisplay> scheduleDisplay;

  const char *parameterFile;
  int termTime;
  BOOL termTimeSpecified;
}

-setParameterFile: (const char *)filename;
-setWealthPNGSave: (int)value fileStem: (const char *)stem;
-setTermTime: (int)terminationTime;
-setBlackAndWhiteRasters: (BOOL)value;
-setScheduleGraphFileOutput: (const char *)filename;
-buildObjects;
-buildActions;
-activateIn: swarmContext;
-(double)countScratchZone;
-(double)countObserverZone;
-(double)countModelZone;
-(double)countDoubleSimples;
-(unsigned long long)countZone: (id <Zone>)z;

@end
