/*
    CharityWorld: SimpleAgent.h
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

The SimpleAgent gives when its wealth is more than the local average,
and begs when the wealth is less than the local average. It always
responds to any begging requests, and always accepts any offer. It
begs from any neighbour with more than average wealth, and offers to
any neighbour with less than average wealth. It gives a fixed amount
(the generosity parameter) and begs for a fixed amount (the beg
parameter).

*/

#import "Agent.h"
#import <collections/List.h>

@class DoubleSimple;

@interface SimpleAgent: Agent {
}

-(BOOL)beggarCondition;		// When I see myself as a begger
-(BOOL)donorCondition;		// When I see myself as a donor
-(BOOL)respondingCondition: (Agent *)nbr;
				// When I respond to a begging request from nbr
-(BOOL)acceptingCondition: (Agent *)nbr;
				// When I accept an offer from nbr
-(BOOL)offeringCondition: (Agent *)nbr;
				// When I offer to nbr
-(BOOL)askingCondition: (Agent *)nbr;
				// When I ask nbr for some money
-(BOOL)happinessCondition;	// When I see myself happy
-step;
-ask;
-offer;
-(void)accept: (DoubleSimple *)amount from: (Agent *)a;
-(void)respondTo: (Agent *)a askingFor: (DoubleSimple *)amount;
-(void)reschedule;

@end
